@@ -25,6 +25,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: sipep.h,v $
+ * Revision 2.46.2.12  2007/01/15 22:16:42  dsandras
+ * Backported patches improving stability from HEAD to Phobos.
+ *
+ * Revision 2.46.2.11  2007/01/02 15:26:46  dsandras
+ * Added DNS Fallback for realm authentication if the classical comparison
+ * doesn't work. Fixes problems with broken SIP proxies. (Ekiga report #377346)
+ *
+ * Revision 2.46.2.10  2006/10/08 13:25:48  dsandras
+ * Fixed previous commit.
+ *
  * Revision 2.46.2.9  2006/08/07 19:46:19  dsandras
  * Backported fix from HEAD to abort registration after a given amount of failures.
  *
@@ -275,7 +285,9 @@ class SIPInfo : public PSafeObject
   public:
     SIPInfo(
       SIPEndPoint & ep, 
-      const PString & name
+      const PString & name,
+      const PTimeInterval & retryMin = PMaxTimeInterval,
+      const PTimeInterval & retryMax = PMaxTimeInterval
     );
 
     ~SIPInfo();
@@ -323,6 +335,9 @@ class SIPInfo : public PSafeObject
     virtual void SetAuthUser(const PString & u)
     { authUser = u;}
     
+    virtual PString GetAuthUser() const
+    { return authUser;}
+
     virtual void SetPassword(const PString & p)
     { password = p;}
     
@@ -356,15 +371,18 @@ class SIPInfo : public PSafeObject
       SIPURL             registrationAddress;
       PString            registrationID;
       SIPTransactionList registrations;
-      PTime	         registrationTime;
+      PTime              registrationTime;
       BOOL               registered;
-      int	         expire;
-      PString	         authRealm;
+      int	               expire;
+      PString	           authRealm;
       PString            authUser;
       PString 	         password;
-      PString		 body;
+      PString		         body;
       PMutex             transportMutex;
       unsigned           authenticationAttempts;
+
+      PTimeInterval retryTimeoutMin; 
+      PTimeInterval retryTimeoutMax; 
     
     private:
 };
@@ -374,7 +392,16 @@ class SIPRegisterInfo : public SIPInfo
   PCLASSINFO(SIPRegisterInfo, SIPInfo);
 
   public:
-    SIPRegisterInfo(SIPEndPoint & ep, const PString & adjustedUsername, const PString & authName, const PString & password, int expire);
+    SIPRegisterInfo(
+      SIPEndPoint & ep, 
+      const PString & originalHost,
+      const PString & adjustedUsername, 
+      const PString & authName, 
+      const PString & password, 
+      int expire, 
+      const PTimeInterval & minRetryTime, 
+      const PTimeInterval & maxRetryTime
+    );
     ~SIPRegisterInfo();
     virtual SIPTransaction * CreateTransaction(OpalTransport &, BOOL);
     virtual SIP_PDU::Methods GetMethod()
@@ -382,6 +409,8 @@ class SIPRegisterInfo : public SIPInfo
 
     virtual void OnSuccess();
     virtual void OnFailed(SIP_PDU::StatusCodes r);
+
+    PString originalHost;
 };
 
 class SIPMWISubscribeInfo : public SIPInfo
@@ -545,8 +574,20 @@ class SIPEndPoint : public OpalEndPoint
   
   /**@name Protocol handling routines */
   //@{
+
+    /**Creates an OpalTransport instance, based on the
+       information provided in address.
+       if isLocalAddress is TRUE, address is interpreted
+       as the local interface to use for outgoing traffic,
+       thus ensuring that the same interface is used
+       for responses as the one where the originating pdu
+       arrived.
+       Else, address is interpreted as the remote address,
+       to which the transport should connect
+      */
     OpalTransport * CreateTransport(
-      const OpalTransportAddress & address
+      const OpalTransportAddress & address,
+      BOOL isLocalAddress = FALSE
     );
 
     virtual void HandlePDU(
@@ -629,7 +670,7 @@ class SIPEndPoint : public OpalEndPoint
     virtual BOOL IsAcceptedAddress(const SIPURL & toAddr);
 
 
-    /**Callback called when MWI is received
+    /**Callback for SIP message received
      */
     virtual void OnMessageReceived (const SIPURL & from,
 				    const PString & body);
@@ -653,7 +694,9 @@ class SIPEndPoint : public OpalEndPoint
       const PString & autName = PString::Empty(),
       const PString & password = PString::Empty(),
       const PString & authRealm = PString::Empty(),
-      int timeout = 0
+      int timeout = 0,
+      const PTimeInterval & minRetryTime = PMaxTimeInterval, 
+      const PTimeInterval & maxRetryTime = PMaxTimeInterval
     );
 
     /**Callback called when MWI is received
@@ -797,7 +840,7 @@ class SIPEndPoint : public OpalEndPoint
     
     /**Return the next CSEQ for the next transaction.
      */
-    unsigned GetNextCSeq() { return ++lastSentCSeq; }
+    unsigned GetNextCSeq() { PWaitAndSignal m(transactionsMutex); return ++lastSentCSeq; }
 
     
     /**Return the SIPAuthentication for a specific realm.
@@ -882,11 +925,6 @@ class SIPEndPoint : public OpalEndPoint
     void SetNATBindingRefreshMethod(const NATBindingRefreshMethod m) { natMethod = m; }
 
 
-  protected:
-    PDECLARE_NOTIFIER(PThread, SIPEndPoint, TransportThreadMain);
-    PDECLARE_NOTIFIER(PTimer, SIPEndPoint, NATBindingRefresh);
-    PDECLARE_NOTIFIER(PTimer, SIPEndPoint, RegistrationRefresh);
-
     /** This dictionary is used both to contain the active and successful
       * registrations, and subscriptions. Currently, only MWI subscriptions
       * are supported.
@@ -921,13 +959,20 @@ class SIPEndPoint : public OpalEndPoint
 	    /**
 	     * Find the SIPInfo object with the specified authRealm
 	     */
-	    SIPInfo *FindSIPInfoByAuthRealm (const PString & authRealm, const PString & userName, PSafetyMode m)
-	    {
-	      for (PSafePtr<SIPInfo> info(*this, m); info != NULL; ++info)
-		      if (authRealm == info->GetAuthentication().GetAuthRealm() && (userName.IsEmpty() || userName == info->GetAuthentication().GetUsername()))
-		        return info;
-	      return NULL;
-	    }
+            SIPInfo *FindSIPInfoByAuthRealm (const PString & authRealm, const PString & userName, PSafetyMode m)
+            {
+              PIPSocket::Address realmAddress;
+
+              for (PSafePtr<SIPInfo> info(*this, m); info != NULL; ++info)
+                if (authRealm == info->GetAuthentication().GetAuthRealm() && (userName.IsEmpty() || userName == info->GetAuthentication().GetUsername()))
+                  return info;
+              for (PSafePtr<SIPInfo> info(*this, m); info != NULL; ++info) {
+                if (PIPSocket::GetHostAddress(info->GetAuthentication().GetAuthRealm(), realmAddress))
+                  if (realmAddress == PIPSocket::Address(authRealm) && (userName.IsEmpty() || userName == info->GetAuthentication().GetUsername()))
+                    return info;
+              }
+              return NULL;
+            }
 
 	    /**
 	     * Find the SIPInfo object with the specified URL. The url is
@@ -954,13 +999,35 @@ class SIPEndPoint : public OpalEndPoint
 	    {
 	      OpalTransportAddress addr = name;
 	      for (PSafePtr<SIPInfo> info(*this, m); info != NULL; ++info) {
-		      if (info->IsRegistered() && (name == info->GetRegistrationAddress().GetHostName() || (info->GetTransport() && addr.GetHostName() == info->GetTransport()->GetRemoteAddress().GetHostName())) && meth == info->GetMethod())
-			return info;
+                if (info->IsRegistered() && (name == info->GetRegistrationAddress().GetHostName() || (info->GetTransport() && addr.GetHostName() == info->GetTransport()->GetRemoteAddress().GetHostName())) && meth == info->GetMethod())
+                  return info;
 	      }
 	      return NULL;
 	    }
     };
 
+    RegistrationList & GetActiveSIPInfo() 
+    { return activeSIPInfo; }
+
+    /** Allow applications to create ancestors of the various registration types
+      */
+    virtual SIPRegisterInfo * CreateRegisterInfo(
+      const PString & originalHost,
+      const PString & adjustedUsername, 
+      const PString & authName, 
+      const PString & password, 
+      int expire, 
+      const PTimeInterval & minRetryTime, 
+      const PTimeInterval & maxRetryTime
+    );
+    virtual SIPMWISubscribeInfo * CreateMWISubscribeInfo(const PString & adjustedUsername, int expire);
+    virtual SIPMessageInfo *      CreateMessageInfo(const PString & adjustedUsername, const PString & body);
+
+  protected:
+    PDECLARE_NOTIFIER(PThread, SIPEndPoint, TransportThreadMain);
+    PDECLARE_NOTIFIER(PTimer, SIPEndPoint, NATBindingRefresh);
+    PDECLARE_NOTIFIER(PTimer, SIPEndPoint, RegistrationRefresh);
+
     static BOOL WriteSIPInfo(
       OpalTransport & transport, 
       void * info
@@ -974,7 +1041,9 @@ class SIPEndPoint : public OpalEndPoint
       const PString & password = PString::Empty(), 
       const PString & authRealm = PString::Empty(),
       const PString & body = PString::Empty(),
-      int timeout = 0
+      int timeout = 0,
+      const PTimeInterval & minRetryTime = PMaxTimeInterval, 
+      const PTimeInterval & maxRetryTime = PMaxTimeInterval
     );
 
     BOOL TransmitSIPUnregistrationInfo (
