@@ -25,6 +25,17 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: patch.cxx,v $
+ * Revision 2.19.2.8  2007/02/10 21:48:35  dsandras
+ * Fixed potential deadlock if ReadPacket takes time to return or does not
+ * return. Thanks to Hannes Friederich for the proposal and the SUN Team
+ * for the bug report (Ekiga #404904).
+ *
+ * Revision 2.19.2.7  2007/01/15 22:16:43  dsandras
+ * Backported patches improving stability from HEAD to Phobos.
+ *
+ * Revision 2.19.2.6  2006/10/06 08:18:16  dsandras
+ * Backported fix from HEAD.
+ *
  * Revision 2.19.2.5  2006/08/07 20:18:25  dsandras
  * Backported various patches from HEAD.
  *
@@ -141,7 +152,7 @@
 /////////////////////////////////////////////////////////////////////////////
 
 OpalMediaPatch::OpalMediaPatch(OpalMediaStream & src)
-  : PThread(10000,
+  : PThread(65536,  //16*4kpage size
             NoAutoDeleteThread,
             HighestPriority,
             "Media Patch:%x"),
@@ -153,6 +164,7 @@ OpalMediaPatch::OpalMediaPatch(OpalMediaStream & src)
 
 OpalMediaPatch::~OpalMediaPatch()
 {
+  inUse.Wait();
   PTRACE(3, "Patch\tMedia patch thread " << *this << " destroyed.");
 }
 
@@ -179,7 +191,7 @@ void OpalMediaPatch::PrintOn(ostream & strm) const
     }
 
     inUse.Signal();
-  }
+  } 
 }
 
 
@@ -189,10 +201,12 @@ void OpalMediaPatch::Main()
   PINDEX i;
 
   inUse.Wait();
+  BOOL isSynchronous = source.IsSynchronous();
   if (!source.IsSynchronous()) {
     for (i = 0; i < sinks.GetSize(); i++) {
       if (sinks[i].stream->IsSynchronous()) {
         source.EnableJitterBuffer();
+        isSynchronous = TRUE;
         break;
       }
     }
@@ -200,18 +214,39 @@ void OpalMediaPatch::Main()
 
   inUse.Signal();
   RTP_DataFrame sourceFrame(source.GetDataSize());
-  while (source.ReadPacket(sourceFrame)) {
-    inUse.Wait();
-    FilterFrame(sourceFrame, source.GetMediaFormat());
+  RTP_DataFrame emptyFrame(source.GetDataSize());
 
-    for (i = 0; i < sinks.GetSize(); i++)
-      sinks[i].WriteFrame(sourceFrame);  // Got write error, remove from sink list
+  while (source.IsOpen()) {
+    if (!source.ReadPacket(sourceFrame))
+      break;
+
+    inUse.Wait();
+    
+    if(!source.IsOpen() || sinks.GetSize() == 0) {
+      inUse.Signal();
+      break;
+    }
+
+    FilterFrame(sourceFrame, source.GetMediaFormat());    
 
     PINDEX len = sinks.GetSize();
+    for (i = 0; i < len; i++)
+      sinks[i].WriteFrame(sourceFrame);  // Got write error, remove from sink list
+
     inUse.Signal();
-    Sleep(5); // Permit to another thread to take the mutex
+
+    if (!isSynchronous || !sourceFrame.GetPayloadSize())
+      Sleep(5); // Don't starve the CPU
+#if !defined(WIN32)
+    else
+      Sleep(5); // Permit to another thread to take the mutex
+#endif
+
     if (len == 0)
       break;
+
+    // make a new, clean frame, so that silence frame won't confuse RFC2833 handler
+    sourceFrame = emptyFrame;
   }
 
   PTRACE(3, "Patch\tThread ended for " << *this);
@@ -227,7 +262,6 @@ void OpalMediaPatch::Close()
   filters.RemoveAll();
   source.Close();
 
-  // This relies on the channel close doing a RemoveSink() call
   while (sinks.GetSize() > 0) {
     OpalMediaStream * stream = sinks[0].stream;
     stream->GetDeleteMutex().Wait();
@@ -351,9 +385,15 @@ OpalMediaPatch::Sink::~Sink()
 
 void OpalMediaPatch::AddFilter(const PNotifier & filter, const OpalMediaFormat & stage)
 {
-  inUse.Wait();
+   PWaitAndSignal mutex(inUse);
+      
+   // ensures that a filter is added only once
+   for (PINDEX i = 0; i < filters.GetSize(); i++) {
+     if (filters[i].notifier == filter && filters[i].stage == stage) {
+       return;
+     }
+   }
   filters.Append(new Filter(filter, stage));
-  inUse.Signal();
 }
 
 
